@@ -22,6 +22,7 @@ import GHC.Generics as GHC
 import Generics.SOP.NP as SOP
 import Generics.SOP.NS as SOP
 import Generics.SOP.BasicFunctors as SOP
+import Generics.SOP.Constraint as SOP
 import Generics.SOP.Metadata as SOP
 import Generics.SOP.Sing
 
@@ -42,7 +43,7 @@ type instance ToSumCode (M1 C c a) xs = ToProductCode a '[] ': xs
 data InfoProxy (c :: *) (f :: * -> *) (x :: *) = InfoProxy
 
 class GDatatypeInfo' (a :: * -> *) where
-  gDatatypeInfo' :: Proxy a -> DatatypeInfo (ToSumCode a '[])
+  gDatatypeInfo' :: proxy a -> DatatypeInfo (ToSumCode a '[])
 
 #if !(MIN_VERSION_base(4,7,0))
 
@@ -55,12 +56,12 @@ isNewtype _ = False
 
 #endif
 
-instance (SingI (ToSumCode a '[]), Datatype c, GConstructorInfos a) => GDatatypeInfo' (M1 D c a) where
+instance (All SListI (ToSumCode a '[]), Datatype c, GConstructorInfos a) => GDatatypeInfo' (M1 D c a) where
   gDatatypeInfo' _ =
     let adt = ADT     (moduleName p) (datatypeName p)
         ci  = gConstructorInfos (Proxy :: Proxy a) Nil
     in if isNewtype p
-       then case isNewtypeShape sing ci of
+       then case isNewtypeShape ci of
               NewYes c -> Newtype (moduleName p) (datatypeName p) c
               NewNo    -> adt ci -- should not happen
        else adt ci
@@ -72,16 +73,16 @@ data IsNewtypeShape (xss :: [[*]]) where
   NewYes :: ConstructorInfo '[x] -> IsNewtypeShape '[ '[x] ]
   NewNo  :: IsNewtypeShape xss
 
-isNewtypeShape :: Sing xss -> NP ConstructorInfo xss -> IsNewtypeShape xss
-isNewtypeShape SCons (x :* Nil) = go shape x
+isNewtypeShape :: All SListI xss => NP ConstructorInfo xss -> IsNewtypeShape xss
+isNewtypeShape (x :* Nil) = go shape x
   where
     go :: Shape xs -> ConstructorInfo xs -> IsNewtypeShape '[ xs ]
     go (ShapeCons ShapeNil) c   = NewYes c
     go _                    _   = NewNo
-isNewtypeShape _     _          = NewNo
+isNewtypeShape _          = NewNo
 
 class GConstructorInfos (a :: * -> *) where
-  gConstructorInfos :: Proxy a -> NP ConstructorInfo xss -> NP ConstructorInfo (ToSumCode a xss)
+  gConstructorInfos :: proxy a -> NP ConstructorInfo xss -> NP ConstructorInfo (ToSumCode a xss)
 
 instance (GConstructorInfos a, GConstructorInfos b) => GConstructorInfos (a :+: b) where
   gConstructorInfos _ xss = gConstructorInfos (Proxy :: Proxy a) (gConstructorInfos (Proxy :: Proxy b) xss)
@@ -89,7 +90,7 @@ instance (GConstructorInfos a, GConstructorInfos b) => GConstructorInfos (a :+:
 instance GConstructorInfos GHC.V1 where
   gConstructorInfos _ xss = xss
 
-instance (Constructor c, GFieldInfos a, SingI (ToProductCode a '[])) => GConstructorInfos (M1 C c a) where
+instance (Constructor c, GFieldInfos a, SListI (ToProductCode a '[])) => GConstructorInfos (M1 C c a) where
   gConstructorInfos _ xss
     | conIsRecord p = Record (conName p) (gFieldInfos (Proxy :: Proxy a) Nil) :* xss
     | otherwise     = case conFixity p of
@@ -102,7 +103,7 @@ instance (Constructor c, GFieldInfos a, SingI (ToProductCode a '[])) => GConstru
       p = InfoProxy
 
 class GFieldInfos (a :: * -> *) where
-  gFieldInfos :: Proxy a -> NP FieldInfo xs -> NP FieldInfo (ToProductCode a xs)
+  gFieldInfos :: proxy a -> NP FieldInfo xs -> NP FieldInfo (ToProductCode a xs)
 
 instance (GFieldInfos a, GFieldInfos b) => GFieldInfos (a :*: b) where
   gFieldInfos _ xs = gFieldInfos (Proxy :: Proxy a) (gFieldInfos (Proxy :: Proxy b) xs)
@@ -156,7 +157,7 @@ instance GProductTo U1 where
 -- This can most certainly be simplified
 class GSumFrom (a :: * -> *) where
   gSumFrom :: a x -> SOP I xss -> SOP I (ToSumCode a xss)
-  gSumSkip :: Proxy a -> SOP I xss -> SOP I (ToSumCode a xss)
+  gSumSkip :: proxy a -> SOP I xss -> SOP I (ToSumCode a xss)
 
 instance (GSumFrom a, GSumFrom b) => GSumFrom (a :+: b) where
   gSumFrom (L1 a) xss = gSumFrom a (gSumSkip (Proxy :: Proxy b) xss)
@@ -234,6 +235,6 @@ gto x = GHC.to (gSumTo x id ((\ _ -> error "inaccessible") :: SOP I '[] -> (GHC.
 -- This is the default definition for 'Generics.SOP.datatypeInfo'.
 -- For more info, see 'Generics.SOP.HasDatatypeInfo'.
 --
-gdatatypeInfo :: forall a. (GDatatypeInfo a) => Proxy a -> DatatypeInfo (GCode a)
+gdatatypeInfo :: forall proxy a. (GDatatypeInfo a) => proxy a -> DatatypeInfo (GCode a)
 gdatatypeInfo _ = gDatatypeInfo' (Proxy :: Proxy (GHC.Rep a))
 
